/**
    Example3a.java  Write then verify an encrypted secret to a SmartDongle.

    MicroWorks, Inc.
    2808 North Cole Road
    Boise, ID 83704

    www.smartdongle.com


    ***** BEGIN LICENSE BLOCK *****

    Version: MPL 1.1/GPL 2.0/LGPL 2.1

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS" basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is MicroWorks, Inc. code.

    The Initial Developer of the Original Code is MicroWorks, Inc. Portions
    created by MicroWorks, Inc. are Copyright (C) 2010
    MicroWorks, Inc. All Rights Reserved.

    Contributor(s):

    Alternatively, the contents of this file may be used under the terms of
    either the GNU General Public License Version 2 or later (the "GPL"), or
    the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
    in which case the provisions of the GPL or the LGPL are applicable instead
    of those above. If you wish to allow use of your version of this file only
    under the terms of either the GPL or the LGPL, and not to allow others to
    use your version of this file under the terms of the MPL, indicate your
    decision by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL or the LGPL. If you do not delete
    the provisions above, a recipient may use your version of this file under
    the terms of any one of the MPL, the GPL or the LGPL.

    ***** END LICENSE BLOCK *****
*/
package com.microworks;

import java.lang.reflect.Array;
import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import com.microworks.SmartDongle;

public class Example3a {

  private static int error;

  // Your keys here
  private final static long p1 = 0xEC6CC589AEFD1E75L;
  private final static long p2 = 0xFCEC0A6A82747B3FL;

  /*
    This is a demo 128 bit encryption key,
    chose some another 16 character key for your project.
  */
  private final static byte[] AESkey = {
    '7',  'o',  'b',  'd',
    'g',  'E',  'n',  '2',
    '\'', 'T',  'H',  '>',
    ']',  'S',  'E',  '|'};

  // Encrypted data stored on SmartDongle.
  private final static String plainText = "This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc";

  private final static int aesBlockSize = 128;

  public static void main(String args[]) {
    try {

      /*
        This instantiates a AES key from the user's key bytes.
        The key is used to initialize a AES cipher for
        encryption and decryption.
      */
      SecretKeySpec skeySpec = new SecretKeySpec(AESkey, "AES");
      Cipher cipher = Cipher.getInstance("AES");
      cipher.init(Cipher.ENCRYPT_MODE, skeySpec);

      // Encrypt the user's application data
      byte[] cipherText = cipher.doFinal(plainText.getBytes());
      int cipherTextLength = Array.getLength(cipherText);

      // Write encrypted data to SmartDongle
      int address = 0;
      error = SmartDongle.Write(p1,
                               p2,
                               address,
                               cipherText,
                               cipherTextLength);
      if (error != 0) {
        System.out.println(
          "Error writing message: " + error + " exiting...");
        return;
      }

      // Verify data
      byte[] input = new byte[cipherTextLength];
      error = SmartDongle.Read(p1,
                              p2,
                              address,
                              input,
                              cipherTextLength);

      int i;
      for (i = 0; i < cipherTextLength; i++) {
        if (input[i] != cipherText[i]) {
          System.out.println("Failed to write encrypted user data, mismatch");
          return;
        }
      }

      System.out.println("Success writing encrypted user data of length "
                         + cipherTextLength);

      // Write AES key to SmartDongle
      error = SmartDongle.Write(p1,
                               p2,
                               address + cipherTextLength,
                               AESkey,
                               aesBlockSize / 8);  // block size is in bits
      if (error != 0) {
        System.out.println("Error AES key: " + error + " exiting...");
        return;
      }

      // Verify that key was written
      error = SmartDongle.Read(p1,
                              p2,
                              address + cipherTextLength,
                              input,
                              aesBlockSize / 8);  // block size is in bits

      for (i = 0; i < aesBlockSize / 8; i++) {
        if (input[i] != AESkey[i]) {
          System.out.println("Failed to write user AES key, mismatch");
          return;
        }
      }

      System.out.println("Success writing AES key of length " +
                         Array.getLength(AESkey));
    }
    catch (java.security.NoSuchAlgorithmException nsafe) {
      System.out.println("Caught Exception:  " + nsafe);
    }
    catch (javax.crypto.NoSuchPaddingException nspe) {
      System.out.println("Caught Exception:  " + nspe);
    }
    catch (javax.crypto.BadPaddingException bpe) {
      System.out.println("Caught Exception:  " + bpe);
    }
    catch (javax.crypto.IllegalBlockSizeException ibse) {
      System.out.println("Caught Exception:  " + ibse);
    }
    catch (java.security.InvalidKeyException ike) {
      System.out.println("Caught Exception:  " + ike);
    }
  }
}
