package com.microworks;



/**
 * @author Ryan Schafer
 */

/**
    SmartDongle.java SmartDongle JNI interface class.

    Headers can be generated from this class using the javah utility included
    in the JDK.

    MicroWorks, Inc.
    2808 North Cole Road
    Boise, ID 83704

    www.smartdongle.com


    ***** BEGIN LICENSE BLOCK *****

    Version: MPL 1.1/GPL 2.0/LGPL 2.1

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS" basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is MicroWorks, Inc. code.

    The Initial Developer of the Original Code is MicroWorks, Inc. Portions
    created by MicroWorks, Inc. are Copyright (C) 2010
    MicroWorks, Inc. All Rights Reserved.

    Contributor(s):

    Alternatively, the contents of this file may be used under the terms of
    either the GNU General Public License Version 2 or later (the "GPL"), or
    the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
    in which case the provisions of the GPL or the LGPL are applicable instead
    of those above. If you wish to allow use of your version of this file only
    under the terms of either the GPL or the LGPL, and not to allow others to
    use your version of this file under the terms of the MPL, indicate your
    decision by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL or the LGPL. If you do not delete
    the provisions above, a recipient may use your version of this file under
    the terms of any one of the MPL, the GPL or the LGPL.

    ***** END LICENSE BLOCK *****
*/

import javax.swing.JOptionPane;

public final class SmartDongle {
	native public static int Read(long p1, long p2, int start,
		byte[] buffer, int size);

	native public static int Write(long p1, long p2, int start,
		byte[] data, int size);

	native public static int LedRed();

	native public static int LedGreen();

	native public static int LedOff();

	native public static int Reset();

	native public static int WriteCryptMode(long p1, long p2, byte mode);

	native public static int ReadCryptMode(byte mode);

	native public static int GetSerialNumber(StringBuffer serial);

	native public static int GetEPROMSize(long p1, long p2);

	native public static int SetStrict();

	native public static String GetErrorString(int error);

	native public static void FileDecrypt(byte[] secret, int scrtLen,
		byte[] key, int blockLen);

	native public static int KeylessRead(int start, byte[] buffer,
		int size);

	native public static int KeylessWrite(int start, byte[] buffer,
		int size);

	native public static int SendKeylessResponse(long response);

	native public static long GetKeylessChallenge();

	static {
	    try {
		    // You can change this path to whatever you want but USK.dll
		    // and usk4j.dll must be present in the directory
		    String libPath = "c:\\smartdongle\\";
		    System.load(libPath + "USK.dll");
		    System.load(libPath + "usk4j.dll");
		}
	    catch(Exception e) {
		JOptionPane.showMessageDialog(null, e.toString());
		e.printStackTrace();
	    }
	}

	/*
	 Begin error constants.
	 */
	public static final int USK_OK = 0;
	/* Can't open dongle */
	public static final int USK_ERR_OPEN = 1;

	/* Dongle is already open. */
	public static final int USK_ERR_IS_OPEN = 2;

	/* Memory allocation error */
	public static final int USK_ERR_MEM = 3;

	/* Deprecated */
	public static final int USK_ERR_EEPROM_SIZE = 4;

	/* Reserved */
	public static final int USK_ERR_5 = 5;

	/* Timeout waiting for state change */
	public static final int USK_ERR_SECURITY_STATE_TIMEOUT = 6;

	/* Timeout waiting for state change */
	public static final int USK_ERR_KEY_STATE_TIMEOUT = 7;

	/* Timeout waiting for TWC to reset */
	public static final int USK_ERR_TWC_RESET_TIMEOUT = 8;

	/* Timeout waiting for task_loop to be flagged as inactive. */
	public static final int USK_ERR_BUSY_TIMEOUT = 9;

	/* The Dongle is Not in ENABLED Mode */
	public static final int USK_ERR_NOT_ENABLED = 10;

	/* Timeout Waiting For Read Flag(s) to be set */
	public static final int USK_ERR_EEPROM_READ_TIMEOUT = 11;

	/* An EEPROM Read ERROR Occurred */
	public static final int USK_ERR_EEPROM_READ = 12;

	/* An EEPROM Write ERROR Occurred */
	public static final int USK_ERR_EEPROM_WRITE = 13;

	/* Dongle failed to validate against user keys */
	public static final int USK_ERR_USK_VALIDATE = 14;

	/* Host failed to validate against dongle keys */
	public static final int USK_ERR_HOST_VALIDATE = 15;

	/* An error occurred when attempting to lock SmartDongle */
	public static final int USK_ERR_DISABLE = 16;

	/* Data format Error */
	public static final int USK_ERR_DATA_FORMAT = 17;

	/* IOCTL's Failed: see sd_user.h */
	public static final int USK_ERR_IOCTL_GET_PIPE_INFO = 18;
	public static final int USK_ERR_IOCTL_GET_DEVICE_DESCRIPTOR = 19;
	public static final int USK_ERR_IOCTL_GET_CONFIG_DESCRIPTOR = 20;
	public static final int USK_ERR_IOCTL_REGISTER_NOTIFY_EVENT = 21;
	public static final int USK_ERR_IOCTL_RESET_DEVICE = 22;
	public static final int USK_ERR_IOCTL_RESET_PIPE = 23;
	public static final int USK_ERR_IOCTL_GET_DRIVER_REVISION = 24;
	public static final int USK_ERR_IOCTL_GET_STRING_DESCRIPTOR = 25;
	public static final int USK_ERR_26 = 26;/* Deprecated */
	public static final int USK_ERR_IOCTL_SEND_KEY = 27;
	public static final int USK_ERR_IOCTL_GET_KEY = 28;
	public static final int USK_ERR_IOCTL_WRITE_PORT = 29;
	public static final int USK_ERR_IOCTL_READ_PORT = 30;
	public static final int USK_ERR_IOCTL_READ_EEPROM = 31;
	public static final int USK_ERR_IOCTL_WRITE_EEPROM = 32;
	public static final int USK_ERR_IOCTL_GET_SECURITY_STATE = 33;
	public static final int USK_ERR_IOCTL_RESET_SECURITY_STATE = 34;
	public static final int USK_ERR_IOCTL_GET_KEY_STATE = 35;
	public static final int USK_ERR_IOCTL_RESET_KEY_STATE = 36;
	public static final int USK_ERR_IOCTL_SEND_HOST_VERIFY_REQUEST = 37;
	public static final int USK_ERR_IOCTL_REQUEST_PAGE = 38;
	public static final int USK_ERR_IOCTL_GET_PAGE_FLAGS = 39;
	public static final int USK_ERR_IOCTL_GET_TWC_FLAG = 40;
	public static final int USK_ERR_IOCTL_GET_BUSY_FLAG = 41;
	public static final int USK_ERR_IOCTL_GET_EEPROM_ERROR_FLAG = 42;

	/* 43, 44 Reserved */
	public static final int USK_ERR_43 = 43;
	public static final int USK_ERR_44 = 44;

	/*
	 * Firmware v1.41 and above.
	 */
	public static final int USK_ERR_IOCTL_QTSDONGLE_SEND_PORT = 45;
	public static final int USK_ERR_IOCTL_QTSDONGLE_GET_FLAGS = 46;
	public static final int USK_ERR_IOCTL_SET_MODE = 47;
	public static final int USK_ERR_IOCTL_GET_MODE = 48;

	/*
	 * For multiple SmartDongles, no device found. If argument specifies serial
	 * number, SmartDongle with the given serial number not found, else no
	 * SmartDongles are on the host.
	 */
	public static final int USK_ERR_NODEV = 49;

	public static final int USK_ERR_MUTEX_DEV_CREATE = 54;
	public static final int USK_ERR_MUTEX_DEV_TIMEOUT = 55;
	public static final int USK_ERR_MUTEX_DEV_ABANDONED = 56;
	public static final int USK_ERR_MUTEX_DEV_UNKNOWN = 57;

	public static final int USK_ERR_MUTEX_API_CREATE = 58;
	public static final int USK_ERR_MUTEX_API_TIMEOUT = 59;
	public static final int USK_ERR_MUTEX_API_ABANDONED = 60;
	public static final int USK_ERR_MUTEX_API_UNKNOWN = 61;

	/* One or more arguments are not correct */
	public static final int USK_ERR_BAD_ARGUMENTS = 63;

	/* Error creating mutex Access Control List */
	public static final int USK_ERR_SID_RETRIEVE = 65;
	public static final int USK_ERR_ACL_INIT = 66;
	public static final int USK_ERR_ACE_ADD = 67;
	public static final int USK_ERR_SD_INIT = 68;
	public static final int USK_ERR_SD_SET = 69;

	/* For SmartDongle v1.50 */
	/* 80 Reserved */
	public static final int USK_ERR_80 = 80;
	public static final int USK_ERR_IOCTL_QTSDONGLE_SET_STRICT = 81;

	/*
	 * An attempt to use extended driver functionality with old driver.
	 */
	public static final int USK_ERR_REQUIRES_DRIVER_2_3_0 = 82;

	/*
	 * An attempt to use extended device functionality with old SmartDongle.
	 */
	public static final int USK_ERR_REQUIRES_DONGLE_1_50 = 83;

	/*
	 * A warning that data written to SmartDongle after a mode change was not on
	 * 16 byte boundaries. (i.e. 0x00, 0x10, 0x20 etc.).
	 *
	 * Ignore this error if all data written on page where boundary violaton
	 * occurred is written with SmartDongle set to the same mode.
	 *
	 * ReadX() and WriteX() user options:
	 *
	 * QTS_OPTION_ON_CHIP_EPROM_PROTECT_DISABLE
	 * QTS_OPTION_ON_CHIP_EPROM_PROTECT_FAST
	 * QTS_OPTION_ON_CHIP_EPROM_PROTECT_STRONG
	 */
	public static final int USK_WARN_MODE_PAGE_BOUNDARY = 84;

	/*
	 * A warning that encrypted data written to SmartDongle was not on 16 byte
	 * boundaries. (i.e. 0x00, 0x10, 0x20 etc.).
	 *
	 * If if the user opts for AES encrytion when writing to all of user space,
	 * this will not be a problem.
	 *
	 * ReadX() and WriteX user option:
	 *
	 * QTS_OPTION_CRYPT_IO
	 */
	public static final int USK_WARN_CRYPT_PAGE_BOUNDARY = 85;

	public static final int USK_RESET_COMEREADY = 86;

	public static final int USK_ERR_JNI = 254;

	public static final int USK_ERR_UNKNOWN = 255;
}
