// usk4j.cpp : JNI Interface for USK.dll
// 
// To build this project you must have a JDK installed, and the proper paths set in the C++ directories.
// It should build with JDK 5 or later.
//
#include <jni.h>
#include "stdafx.h"

#define USK_ERR_JNI 254

JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_Read
  (JNIEnv *env, jclass c, jlong p1, jlong p2, jint start, jbyteArray data, jint length)
{
	unsigned char *ndata = new unsigned char[length];
	ZeroMemory(ndata, length);
	int ret = Read(p1, p2, start, ndata, length);
	env->SetByteArrayRegion(data, 0, length, (jbyte*)ndata);
	
	return (jint)ret;
}

JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_KeylessRead
  (JNIEnv *env, jclass c, jint start, jbyteArray data, jint length)
{
	unsigned char *ndata = new unsigned char[length];
	ZeroMemory(ndata, length);
	int ret = KeylessRead(start, ndata, length);
	env->SetByteArrayRegion(data, 0, length, (jbyte*)ndata);
	
	return (jint)ret;
}

JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_KeylessWrite
  (JNIEnv *env, jclass c, jint start, jbyteArray data, jint length)
{
	jsize size = env->GetArrayLength(data);
	unsigned char *ndata = new unsigned char[size];
	ndata = (unsigned char*)env->GetByteArrayElements(data, false);
	
	int ret = KeylessWrite(start, ndata, length);

	return (jint)ret;
}

JNIEXPORT jlong JNICALL Java_com_microworks_SmartDongle_GetKeylessChallenge
  (JNIEnv *env, jclass c)
{
	unsigned __int64 ret = GetKeylessChallenge();
	return (jlong)ret;
}

JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_SendKeylessResponse
  (JNIEnv *env, jclass c, jlong response)
{
	int ret = SendKeylessResponse(response);
	return (jint)ret;
}

JNIEXPORT void JNICALL Java_com_microworks_SmartDongle_FileDecrypt
  (JNIEnv *env, jclass c, jbyteArray secret, jint scrtLen, jbyteArray key, jint blockLen)
{
	int size = ((int)scrtLen*(int)blockLen)/8;
	unsigned char *ndata = new unsigned char[size];
	ndata = (unsigned char*)env->GetByteArrayElements(secret, false);
	unsigned char *myKey = new unsigned char[16];
	myKey = (unsigned char*)env->GetByteArrayElements(key, false);
	FileDecrypt(ndata, scrtLen, myKey, blockLen);
	env->SetByteArrayRegion(secret, 0, size, (jbyte*)ndata);
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    SmartDongleWrite
 * Signature: (JJI[BI)I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_Write
  (JNIEnv *env, jclass c, jlong p1, jlong p2, jint start, jbyteArray data, jint length)
{
	jsize size = env->GetArrayLength(data);
	unsigned char *ndata = new unsigned char[size];
	ndata = (unsigned char*)env->GetByteArrayElements(data, false);
	
	int ret = Write(p1, p2, start, ndata, length);

	return (jint)ret;
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    SmartDongleLedRed
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_LedRed
  (JNIEnv *env, jclass c)
{
	return (jint)LedRed();
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    SmartDongleLedGreen
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_LedGreen
  (JNIEnv *env, jclass c)
{
	return (jint)LedGreen();
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    SmartDongleLedOff
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_LedOff
  (JNIEnv *env, jclass c)
{
	return (jint)LedOff();
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    SmartDongleReset
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_Reset
  (JNIEnv *env, jclass c)
{
	return (jint)Reset();
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    WriteCryptMode
 * Signature: (JJB)I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_WriteCryptMode
  (JNIEnv *env, jclass c, jlong p1, jlong p2, jbyte mode)
{
	return (jint)WriteCryptMode(p1, p2, mode);
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    ReadCryptMode
 * Signature: (B)I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_ReadCryptMode
  (JNIEnv *env, jclass c, jbyte mode)
{
	unsigned char m;
	int ret = ReadCryptMode(&m);
	mode = (jbyte)m;
	return (jint)ret;
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    FileDecrypt
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_FileDecrypt
  (JNIEnv *env, jclass c, jbyte mode)
{
	unsigned char m;
	int ret = ReadCryptMode(&m);
	mode = (jbyte)m;
	return (jint)ret;
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    GetSerialNumber
 * Signature: (Ljava/lang/StringBuffer;)I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_GetSerialNumber
  (JNIEnv *env, jclass c, jobject serial)
{
	if (serial == 0) {
		return USK_ERR_JNI;
	}
	jclass clazz = env->GetObjectClass (serial);

	jmethodID mid = env->GetMethodID (clazz, "append", "(Ljava/lang/String;)Ljava/lang/StringBuffer;");
    // If this method does not exist then return.
    if (mid == 0)
            return USK_ERR_JNI;
	char *sn = new char[128];
	ZeroMemory(sn, 128);
	int ret = GetSerialNumber(sn);
	jstring jsn = env->NewStringUTF(sn);

	env->CallObjectMethod(serial, mid, jsn);

	return (jint)ret;
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    GetEEPROMSize
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_GetEPROMSize
  (JNIEnv *env, jclass c, jlong p1, jlong p2)
{
	return (jint)GetEpromSize(p1, p2);
}


/*
 * Class:     com_microworks_SmartDongle
 * Method:    GetErrorString
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_microworks_SmartDongle_GetErrorString
  (JNIEnv *env, jclass c, jint error)
{
	char *err = new char[QTS_ERROR_STRING_LENGTH_MAX];
	GetErrorString(err, error);
	return env->NewStringUTF(err);
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    InitGlobalMutex
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_microworks_SmartDongle_InitGlobalMutex
  (JNIEnv *env, jclass c)
{
	return (jint)InitGlobalMutex();
}

/*
 * Class:     com_microworks_SmartDongle
 * Method:    CloseGlobalMutex
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_com_microworks_SmartDongle_CloseGlobalMutex
  (JNIEnv *env, jclass c)
{
	return (jboolean)CloseGlobalMutex();
}